﻿using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;
using UnityEngine.SceneManagement;

public class Anthill : MonoBehaviour
{
	[System.Serializable]
	public class AnthillLevelConfig
	{
		public float neededMaterial;
		public float maxAnts;
	}

	const float WORLD_SIZE_SCALE_FACTOR = 5.0f;

	public GameObject antSprite, enemyAntSprite;
	public List <GameObject> foodSprites;
	public List <Sprite> foodSpritesForCapacity;
	public GameObject materialSprite;
	public int startAnts = 1;
	public float antOrderRadius = 1;
	public float antScanRadius = 1, antTranslateRadius = 1;
	public int startFood = 100;
	public float antMaxDistanceFactor = 10.0f;
	public float antSpawnInterval = 1.0f;
	public float antDeathInterval = 5.0f;
	public int enemyAntHP = 3;
	public float enemyAntSpawnInterval = 30.0f;
	public int enemyAntSpawnIncrease = 1;
	public float foodConsumptionInterval = 1.0f;
	public float foodConsumptionPerAnt = 0.01f;
	public int startFoodPoints = 30, startMaterialPoints = 30;
	public List <int> foodPointCapacities, materialPointCapacities;
	public List <AnthillLevelConfig> levels;

	private int level = 0;
	private float food = 0, material = 0;

	float worldSize = 0, anthillSize = 0;
	private List <Ant> ants;
	private List <Ant> antsToRemove;
	private List <AntObject> objects;
	private List <AntObject> objectsToRemove;
	private List <EnemyAnt> enemyAnts;
	private List <EnemyAnt> enemyAntsToRemove;
	private List <AntObject> allObjects;
	private float antSpawnTime = 0, antDeathTime = 0;
	private int enemyAntSpawnCount = 0;
	private float enemyAntSpawnTime = 0;
	private float foodConsumptionTime = 0;
	private int voiceCount = 0;

	void Start()
	{
		worldSize = GameObject.Find("Plane").transform.localScale.x * WORLD_SIZE_SCALE_FACTOR;
		anthillSize = transform.localScale.magnitude;

		food = startFood;
		enemyAntSpawnCount = enemyAntSpawnIncrease;

		ants = new List <Ant> ();
		antsToRemove = new List <Ant> ();
		objects = new List <AntObject> ();
		objectsToRemove = new List <AntObject> ();
		enemyAnts = new List <EnemyAnt>	();
		enemyAntsToRemove = new List <EnemyAnt>	();
		allObjects = new List <AntObject> ();

		for (int i = 0; i < startAnts; ++i)
		{
			spawnAnt();
		}

		for (int i = 0; i < startFoodPoints; ++i)
		{
			int capacity = foodPointCapacities[Random.Range(0, foodPointCapacities.Count)];
			Vector3 position = new Vector3();
			bool isCorrectPos = false;
			do
			{
				do
				{
					position = Random.insideUnitSphere * worldSize * (0.3f + ((float) i * 2 / (float) startFoodPoints));
					position.y = 0;
				} while (position.magnitude < anthillSize);
				NavMeshHit hit;
				if (NavMesh.SamplePosition(position, out hit, 1.0f, NavMesh.AllAreas))
				{
					position = hit.position;
					isCorrectPos = true;
				}
			} while (!isCorrectPos);
			int farmTypeIndex = Random.Range(0, foodSprites.Count);
			createFarmPoint(foodSprites[farmTypeIndex], position, Resource.Food, capacity, farmTypeIndex);
		}

		for (int i = 0; i < startMaterialPoints; ++i)
		{
			int capacity = materialPointCapacities[Random.Range(0, materialPointCapacities.Count)];
			Vector3 position = new Vector3();
			bool isCorrectPos = false;
			do
			{
				do
				{
					position = Random.insideUnitSphere * worldSize * (0.3f + ((float) i * 2 / (float) startMaterialPoints));
					position.y = 0;
				} while (position.magnitude < anthillSize);
				NavMeshHit hit;
				if (NavMesh.SamplePosition(position, out hit, 1.0f, NavMesh.AllAreas))
				{
					position = hit.position;
					isCorrectPos = true;
				}
			} while (!isCorrectPos);
			createFarmPoint(materialSprite, position, Resource.Material, capacity);
		}
	}
	
	void Update()
	{
		float dt = Time.deltaTime;
		foreach (Ant ant in ants)
		{
			ant.update(dt);
		}
		
		foreach (EnemyAnt enemyAnt in enemyAnts)
		{
			// second game losing condition - enemy ant reaches ant hill
			Vector3 anthillPos = transform.position;
			Vector3 enemyPos = enemyAnt.getSprite().transform.position;
			if ((anthillPos - enemyPos).magnitude < anthillSize * 0.5f)
			{
				restartGame();
				return;
			}
		}

		foodConsumptionTime += dt;
		while (foodConsumptionTime > foodConsumptionInterval)
		{
			food -= foodConsumptionPerAnt * ants.Count;
			foodConsumptionTime -= foodConsumptionInterval;
		}

		if (food > 0 && food < levels[level].maxAnts)
		{
			antSpawnTime += dt;
			while (antSpawnTime > antSpawnInterval)
			{
				if (ants.Count >= levels[level].maxAnts) break;
				spawnAnt();
				antSpawnTime -= antSpawnInterval;
			}
		}
		else
		{
			food = 0;
			antDeathTime += dt;
			while (antDeathTime > antDeathInterval)
			{
				destroyOldestAnt();
				antDeathTime -= antDeathInterval;
			}
			if (ants.Count == 0)
			{
				restartGame();
			}
		}

		enemyAntSpawnTime += dt;
		if (enemyAntSpawnTime > enemyAntSpawnInterval)
		{
			for (int i = 0; i < enemyAntSpawnCount; ++i)
			{
				Vector3 position = new Vector3();
				bool isCorrectPos = false;
				do
				{
					do
					{
						position = Random.insideUnitSphere * worldSize;
						position.y = 0;
					}
					while (position.magnitude < worldSize * 0.5f);
					NavMeshHit hit;
					if (NavMesh.SamplePosition(position, out hit, 1.0f, NavMesh.AllAreas))
					{
						position = hit.position;
						isCorrectPos = true;
					}
				}
				while (!isCorrectPos);
				createEnemyAnt(position);
			}

			enemyAntSpawnTime %= dt;
			enemyAntSpawnCount += enemyAntSpawnIncrease;
		}

		foreach (Ant ant in antsToRemove)
		{
			ants.Remove(ant);
			allObjects.Remove(ant);
		}
		antsToRemove.Clear();
		foreach (AntObject obj in objectsToRemove)
		{
			objects.Remove(obj);
			allObjects.Remove(obj);
		}
		objectsToRemove.Clear();
		foreach (EnemyAnt enemyAnt in enemyAntsToRemove)
		{
			enemyAnts.Remove(enemyAnt);
			objects.Remove(enemyAnt);
			allObjects.Remove(enemyAnt);
		}
		objectsToRemove.Clear();
	}

	public void spawnAnt()
	{
		Vector3 spawnPos = Random.insideUnitSphere * anthillSize * 3.0f;
		spawnPos.y = 0;
		Ant ant = new Ant(this, antSprite, spawnPos, antScanRadius, antTranslateRadius);
		ants.Add(ant);
		allObjects.Add(ant);
	}

	public void destroyOldestAnt()
	{
		if (ants.Count > 0)
		{
			ants[0].die();
		}
	}

	public void createFarmPoint(GameObject prefab, Vector3 position, Resource resource, float capacity, int farmTypeIndex = 0)
	{
		FarmPoint farmPoint = new FarmPoint(this, prefab, position, farmTypeIndex);
		farmPoint.setResource(resource);
		farmPoint.setCapacity(capacity);
		objects.Add(farmPoint);
		allObjects.Add(farmPoint);
	}

	public void createEnemyAnt(Vector3 position)
	{
		EnemyAnt enemy = new EnemyAnt(this, enemyAntSprite, position, enemyAntHP);
		objects.Add(enemy);
		enemyAnts.Add(enemy);
		allObjects.Add(enemy);
	}

	public void sendAntOrder(Vector3 position)
	{
		foreach (Ant ant in ants)
		{
			Vector3 antPos = ant.getSprite().transform.position;
			if ((position - antPos).magnitude <= antOrderRadius)
			{
				ant.setThought(new OrderThought(position));
				ant.getCloud().setSprite((int)Cloud.Order);
			}
		}
	}

	public AntObject makeHomeObject(Vector3 relativePos)
	{
		Vector3 dirToRelative = (relativePos - transform.position).normalized;
		Vector3 anthillPos = transform.position + dirToRelative * anthillSize * (0.5f + Random.value * 0.5f);
		return new AntObject(this, anthillPos);
	}

	public bool isAntTooFarAway(Ant ant)
	{
		return ((ant.getSprite().transform.position - transform.position).magnitude > anthillSize * antMaxDistanceFactor);
	}

	public int getLevel()
	{
		return level;
	}

	public float getFood()
	{
		return food;
	}

	public void addFood(float food)
	{
		this.food += food;
	}

	public float getMaterial()
	{
		return material;
	}

	public void addMaterial(float material)
	{
		this.material += material;
		if (this.material >= levels[level].neededMaterial)
		{
			++level;
		}
	}

	public Sprite getFoodSpriteForCapacity(int farmTypeIndex, float capacity)
	{
		int capacityIndex = 0;
		for (int i = 1; i < foodPointCapacities.Count; ++i)
		{
			if (capacity > foodPointCapacities[i - 1]) capacityIndex = i;
		}
		return foodSpritesForCapacity[farmTypeIndex * 3 + capacityIndex];
	}

	public List <Ant> getAnts()
	{
		return ants;
	}

	public int getAntsCount()
	{
		return ants.Count;
	}

	public List <AntObject> getObjects()
	{
		return objects;
	}

	public List <AntObject> getAllObjects()
	{
		return allObjects;
	}

	public bool hasObject(AntObject obj)
	{
		return objects.Contains(obj);
	}

	public void removeAnt(Ant ant)
	{
		antsToRemove.Add(ant);
	}

	public void removeEnemyAnt(EnemyAnt enemyAnt)
	{
		enemyAntsToRemove.Add(enemyAnt);
	}

	public void removeObject(AntObject obj)
	{
		objectsToRemove.Add(obj);
	}

	public bool checkVoice()
	{
		return (voiceCount < 10);
	}

	public void addSound()
	{
		voiceCount++;
	}

	public void removeSound()
	{
		voiceCount--;
	}

	public void restartGame()
	{
		SceneManager.LoadScene(0);
	}
}
