﻿using UnityEngine;
using UnityStandardAssets.ImageEffects;

public class CameraControl : MonoBehaviour
{
	const float PAN_ZOOM_FACTOR = 45.0f;

	public float mouseSensitivity = 1.0f;
	public float minCamSize = 10.0f, maxCamSize = 30.0f;
	private Vector3 lastMousePos;
	private bool menu = true;
	public GameObject orderFX;

	Blur blur;
	Anthill anthill;
	Camera cam;
	public Texture logo;

	public Texture anthillIcon;
	public Texture foodIcon;
	public Texture matIcon;
	public Texture antIcon;

	void Start()
	{
		menu = true;
		blur = GetComponent <Blur> ();
		anthill = (Anthill) GameObject.Find ("Anthill").GetComponent <Anthill> ();
		cam = GetComponent <Camera> ();
	}

	void OnGUI()
	{
		if (menu)
		{
			GUI.DrawTexture(new Rect (Screen.width / 2 - 250, Screen.height / 2 - 340, 500, 200), logo, ScaleMode.StretchToFill, true, 10.0F);
			GUI.Label (new Rect (Screen.width / 2 - 150, Screen.height / 2 - 140, 400, 70), "Made by Dividebyzer0 team for GlobalGameJam2017\n"); 


			GUI.Label (new Rect (Screen.width / 2 - 200, Screen.height / 2 - 90, 400, 70), "ErrAnt is a survival RTS about ants with only one type of control - spread the waves of fear. "); 
			GUI.Label (new Rect (Screen.width / 2 - 200, Screen.height / 2 - 30, 400, 70), "Your goal is to maintain an anthill: gather food and resources to grow, and defend it from ravenous neighbors and other dangers. To achieve this, you have to master the wave-like type of communication between ants.\n\n"); 
			GUI.Label (new Rect (Screen.width - 350, Screen.height - 120, 400, 110), "Credits:\nDmitriy \"Stranger\" Troitskiy - project architecture, code\nNikita “teuz” Zhimkov - gamedesign, code\nElla Grischenko - art, animation\nGeorgy \"R0ller\" Yushkevich - gamedesign, audio\nNikita \"DNK\" Pavlov - gamedesign, direction\n\n"); 



			if (GUI.Button (new Rect (Screen.width / 2 - 40, Screen.height / 2 + 40, 90, 30), "Play")) 
			{
				menu = !menu;
				blur.enabled = menu;
			}
			if (GUI.Button (new Rect (Screen.width / 2 - 40, Screen.height / 2 + 80, 90, 30), "Exit")) 
			{
				Application.Quit();
			}
		}
		else 
		{
			
			GUI.DrawTexture (new Rect (10, 10, 15, 15), anthillIcon, ScaleMode.StretchToFill, true);
			GUI.DrawTexture (new Rect (10, 30, 15, 15), foodIcon, ScaleMode.StretchToFill, true);
			GUI.DrawTexture (new Rect (10, 50, 15, 15), matIcon, ScaleMode.StretchToFill, true);
			GUI.DrawTexture (new Rect (10, 70, 15, 15), antIcon, ScaleMode.StretchToFill, true);

			GUI.Label(new Rect (30, 10, 200, 50), "Anthill level: " + (anthill.getLevel() + 1).ToString());
			GUI.Label(new Rect (30, 30, 200, 50), "Food: " + Mathf.Round(anthill.getFood()).ToString());
			GUI.Label(new Rect (30, 50, 200, 50), "Material: " + Mathf.Round(anthill.getMaterial()).ToString());
			GUI.Label(new Rect (30, 70, 200, 50), "Ants: " + anthill.getAntsCount());
		}
	}

	void Update()
	{
		if (Input.GetKeyDown(KeyCode.Escape))
		{
			menu = !menu;
			blur.enabled = menu;
		}

		if (Input.GetKeyDown (KeyCode.Space))
		{
			cam.transform.position = new Vector3 (0f, 10f, 0f);
		}

		if (!menu)
		{
			Camera camera = GetComponent <Camera> ();
			camera.orthographicSize -= Input.GetAxis ("Mouse ScrollWheel") * camera.orthographicSize;
			if (camera.orthographicSize < minCamSize)
			{
				camera.orthographicSize = minCamSize;
			}
			if (camera.orthographicSize > maxCamSize)
			{
				camera.orthographicSize = maxCamSize;
			}

			if (Input.GetMouseButtonDown (1))
			{
				lastMousePos = Input.mousePosition;
			}

			if (Input.GetMouseButton(1))
			{
				Vector3 delta = (lastMousePos - Input.mousePosition) * (camera.orthographicSize / PAN_ZOOM_FACTOR);
				transform.Translate (delta.x * mouseSensitivity, delta.y * mouseSensitivity, 0);
				lastMousePos = Input.mousePosition;
			}

			checkClick ();
		}
	}

	private void checkClick()
	{
		if (Input.GetMouseButton(0))
		{
			Ray ray = cam.ScreenPointToRay(new Vector3(Input.mousePosition.x, Input.mousePosition.y, 0f));
			Debug.DrawRay(ray.origin, ray.direction * 10, Color.red);
			RaycastHit hit;
			if (Physics.Raycast(ray.origin, ray.direction, out hit, 15.0f))
			{
				GameObject.Instantiate (orderFX,new Vector3(hit.point.x,4f,hit.point.z),Quaternion.identity);
				anthill.sendAntOrder(hit.point);
			}
		}
	}
}
