﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CloudThought : MonoBehaviour {
	public Transform parent;
	SpriteRenderer spriteRenderer;
	public Sprite[] cloudSprites;
	Anthill anthill;

	public AudioClip[] farmVoices;
	public AudioClip[] idleVoices;
	public AudioClip[] enemyVoices;
	public AudioClip[] alarmVoices;

	private float alpha = 1f;
	private float roarCD=5f;
	private float removeSound=-1f;
	AudioSource aud;
	int state = 0;

	void Start()
	{
		anthill = GameObject.Find ("Anthill").GetComponent<Anthill> ();
		InitRoar ();
		aud = (AudioSource)GetComponent<AudioSource> ();
		alpha = 1f;
		spriteRenderer = (SpriteRenderer)GetComponent<SpriteRenderer> ();
	}

	void checkRemoveSound()
	{
		removeSound -= Time.deltaTime;
		if (removeSound < 0)
			anthill.removeSound();

	}


	void Update () {
		
		if (removeSound > 0)
			checkRemoveSound ();

		roarCD -= Time.deltaTime;
		if (roarCD < 0)
			Roar (state);
		if (alpha > 0) alpha -= Time.deltaTime/3;
		spriteRenderer.color = new Color(spriteRenderer.color.r,spriteRenderer.color.g,spriteRenderer.color.b,alpha);
		transform.rotation = Quaternion.LookRotation(Vector3.down , Vector3.forward);
		transform.position = parent.position + new Vector3 (1f, 0f, 1f);
	}

	public void setSprite(int cloud)
	{
		
		if (cloud == (int)Cloud.Order && Random.Range(0f,1f) < 0.4f)
		//if (cloud != state)
				roarCD = Random.Range(0f,0.3f);
		state = cloud;
		if (spriteRenderer.sprite != cloudSprites [cloud])
			alpha = 1f;
		spriteRenderer.sprite = cloudSprites[cloud];
	}

	void InitRoar()
	{
		roarCD = Random.Range (30f, 180f);
	}

	void Roar(int state)
	{
		if (anthill.checkVoice ()) 
		{
			anthill.addSound ();
			aud.clip = getRoar (state);
			aud.Play ();
			removeSound = 1f;
		}
		InitRoar ();
			
		
	}

	AudioClip getRoar(int state)
	{
		switch (state) 
		{
		case (int)Cloud.Eat:
		case (int)Cloud.Mat:
		case (int)Cloud.Home:
			return farmVoices [Random.Range (0, farmVoices.Length - 1)];
		case (int)Cloud.Enemy:
			return enemyVoices [Random.Range (0, enemyVoices.Length - 1)];
		case (int)Cloud.Order:
			return alarmVoices [Random.Range (0, alarmVoices.Length - 1)];
		case (int)Cloud.Quest:
			return idleVoices [Random.Range (0, idleVoices.Length - 1)];
		default:
			return null;
		}
	}

}
