﻿using UnityEngine;

public class DangerThought : Thought
{
	const int BASE_RATING = 60;
	const float BASE_EXPIRATION_TIME = 5.0f;
	const float RUNAWAY_DISTANCE = 5.0f;

	public DangerThought(AntObject target) : base(target)
	{
		rating = BASE_RATING;
		expirationTime = BASE_EXPIRATION_TIME;
		this.target = target;
		cloudType = (int)Cloud.Enemy;
	}

	public DangerThought(AntObject target, int rating, float expirationTime) : base(target, rating, expirationTime)
	{
		this.rating = rating;
		this.expirationTime = expirationTime;
		this.target = target;
		cloudType = (int)Cloud.Enemy;
	}

	public override Thought clone()
	{
		return new DangerThought(target, rating, expirationTime);
	}

	override public void think(Ant ant, float dt)
	{
		if (!ant.getAnthill().hasObject(target))
		{
			ant.setThought(null);
			return;
		}

		Vector3 antPos = ant.getSprite().transform.position;
		ant.move(target.getSprite().transform.position);
		if (ant.hasReachedTarget())
		{
			ant.sacrifice((EnemyAnt) target);
			ant.setThought(null);
			return;
		}

		checkExpiraton(ant, dt);
	}
}
