﻿using System;
using UnityEngine;

public class FarmThought : Thought
{
	const int BASE_RATING = 10;
	const float BASE_EXPIRATION_TIME = 30.0f;

	public FarmThought(AntObject target) : base(target)
	{
		rating = BASE_RATING;
		expirationTime = BASE_EXPIRATION_TIME;
		cloudType = (int) Cloud.Eat;
	}

	public FarmThought(AntObject target, int rating, float expirationTime) : base(target, rating, expirationTime)
	{
		FarmPoint fp = (FarmPoint)target;

		cloudType = (fp.getResource() == Resource.Food) ?  (int) Cloud.Eat : (int)Cloud.Mat;
	}

	public override Thought clone()
	{
		return new FarmThought(target, rating, expirationTime);
	}

	public override bool shouldTranslate(Ant ant, int compareRating)
	{
		bool result =  base.shouldTranslate(ant, compareRating);
		if (result && ant.getResource() != Resource.None) return false;
		return result;
	}

	override public void think(Ant ant, float dt)
	{
		// checking that target is still existing object
		Anthill anthill = ant.getAnthill();
		if (!anthill.hasObject(target))
		{
			ant.setThought(null);
			return;
		}

		Vector3 antPos = ant.getSprite().transform.position;
		Vector3 targetPos = (target.getSprite() != null ? target.getSprite().transform.position : target.getPosition());
		if (ant.getResource() == Resource.None)
		{
			ant.move(targetPos);
			if (ant.hasReachedTarget())
			{
				FarmPoint farmPoint = (FarmPoint) target;
				if (farmPoint.farm() > 0)
				{
					ant.setResource(farmPoint.getResource());
					ant.setThought(new HomeThought(anthill.makeHomeObject(antPos), this));
				}
			}
		}
		else
		{
			if (!(ant.getThought() is HomeThought))
			{
				ant.setThought(new HomeThought(anthill.makeHomeObject(antPos), this));
			}
		}

		checkExpiraton(ant, dt);
	}
}
