﻿using UnityEngine;

class HomeThought : Thought
{
	const int BASE_RATING = 10;

	public HomeThought(AntObject target, Thought lastThought) : base(target)
	{
		rating = BASE_RATING;
		this.lastThought = lastThought;
		// HomeThought has no expiration time
		cloudType = (int) Cloud.Home;
	}

	public HomeThought(AntObject target, Thought lastThought, int rating) : base(target, rating, Mathf.Infinity)
	{
		this.lastThought = lastThought;
		cloudType = (int) Cloud.Home;
	}

	public override Thought clone()
	{
		return new HomeThought(target, lastThought, rating);
	}

	override public bool isTranslateable()
	{
		return false;
	}

	override public void think(Ant ant, float dt)
	{
		Vector3 targetPos = (target.getSprite() != null ? target.getSprite().transform.position : target.getPosition());
		ant.move(targetPos);
		if (ant.hasReachedTarget())
		{
			switch (ant.getResource())
			{
				case Resource.Food:
					ant.getAnthill().addFood(1.0f);
				break;
				case Resource.Material:
					ant.getAnthill().addMaterial(1.0f);
				break;
			}
			ant.setResource(Resource.None);
			ant.setThought(lastThought);
		}
	}

	private Thought lastThought;
}
