﻿using UnityEngine;

class OrderThought : Thought
{
	const int BASE_RATING = 20;
	const float BASE_EXPIRATION_TIME = 1.0f;
	const float RUNAWAY_DISTANCE = 5.0f;

	public OrderThought(Vector3 position) : base(null)
	{
		this.position = position;
		this.rating = BASE_RATING;
		this.expirationTime = BASE_EXPIRATION_TIME;
		this.cloudType = (int)Cloud.Order;
	}

	public OrderThought(Vector3 position, int rating, float expirationTime) : base(null, rating, expirationTime)
	{
		this.position = position;
		this.rating = rating;
		this.expirationTime = expirationTime;
		this.cloudType = (int)Cloud.Order;
	}

	public override Thought clone()
	{
		return new OrderThought(position, rating, expirationTime);
	}

	override public void think(Ant ant, float dt)
	{
		Vector3 antPos = ant.getSprite().transform.position;
		Vector3 runawayPos = antPos + (antPos - position).normalized * RUNAWAY_DISTANCE;
		ant.move(runawayPos);
		if (ant.hasReachedTarget())
		{
			// removing danger and resetting thought
			ant.getAnthill().removeObject(target);
			ant.setThought(null);
		}

		checkExpiraton(ant, dt);
	}

	private Vector3 position;
}
