﻿using UnityEngine;

abstract public class Thought
{
	const int BASE_RATING = 10;
	const float BASE_EXPIRATION_TIME = 10;
	protected int rating; // thought importance rating
	protected float expirationTime; // thought lifetime
	protected AntObject target;
	protected int cloudType;

	public Thought(AntObject target)
	{
		this.target = target;
		rating = BASE_RATING;
		expirationTime = BASE_EXPIRATION_TIME;
	}

	public Thought(AntObject target, int rating, float expirationTime)
	{
		this.target = target;
		this.rating = rating;
		this.expirationTime = expirationTime;
	}

	abstract public Thought clone();

	virtual public bool isTranslateable()
	{
		return true;
	}

	virtual public bool shouldTranslate(Ant ant, int compareRating)
	{
		if (rating > compareRating) return true;

		float crP3 = Mathf.Pow(compareRating, 3);
		float probability = crP3 / (Mathf.Pow(rating, 3) + crP3);
		return Random.value < probability;
	}

	virtual public void think(Ant ant, float dt)
	{
	}

	protected void checkExpiraton(Ant ant, float dt)
	{
		expirationTime -= dt;
		if (expirationTime <= 0f) 
		{
			ant.setThought(null);
			ant.getCloud ().setSprite ((int)Cloud.Quest);
		}
	}

	public int getRating()
	{
		return rating;
	}

	public int getCloudType ()
	{
		return cloudType;
	}
}
